#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Indigo Sunsynk Inverter plugin
# Copyright (c) 2022 Alan Carter. All rights reserved.
#

################################################################################
# Imports
################################################################################

from sunsynk.client import SunsynkClient


################################################################################
# Globals
################################################################################

def updateVar(name, value, folder=0):
    if name not in indigo.variables:
        indigo.variable.create(name, value=value, folder=folder)
    else:
        indigo.variable.updateValue(name, value)

################################################################################
class Plugin(indigo.PluginBase):
    ########################################
    # Class properties
    ########################################
    
    ########################################
    def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs): 
        indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
        self.userId = pluginPrefs.get("sunsynkId", "")
        self.passWord = pluginPrefs.get("sunsynkPw", "")
        self.logComms = pluginPrefs.get("logComms", False)
    
    ########################################
    def __del__(self):
        indigo.PluginBase.__del__(self)
        
    ########################################
    def startup(self):
        indigo.server.log("Starting Sunsynk plugin")
        self.commsEnable = True
        self.retries = 0
        self.startComms()
        self.createDevices()
        
    def deviceStartComm(self, dev):
        dev.stateListOrDisplayStateIdChanged()
        return

    def shutdown(self):
        indigo.server.log("Stopping Sunsynk plugin")
        self.commsEnable = False
        pass
        
    ########################################
    def runConcurrentThread(self):
        indigo.server.log("Starting Sunsynk monitoring thread")

        try:
            while True:
                self.sleep(30)
                self.getReadings()
                        
        except self.StopThread:
            self.commsEnable = False


    ########################################
    # Called when user prefs are changed
    
    def closedPrefsConfigUi(self, valuesDict, userCancelled):
        # If the user saved preferences, update logging parameters
        if userCancelled == False:
            logComms = self.logComms
            self.logComms = valuesDict.get("logComms", False)
            if self.logComms != logComms:
                if self.logComms:
                    indigo.server.log("Comms logging is on")
                else:
                    indigo.server.log("Comms logging is off")

        
    ########################################
    # Sunsynk specific functions

            
    def startComms(self):
        if self.commsEnable:
            self.client = SunsynkClient(self.userId, self.passWord)

            
    def createDevices(self):
        inverters = self.client.get_inverters()
        if self.logComms:
            indigo.server.log("Inverter:")
            indigo.server.log(str(inverters))
        for inverter in inverters:
            grid = self.client.get_inverter_realtime_grid(inverter.sn)
            battery = self.client.get_inverter_realtime_battery(inverter.sn)
            solar = self.client.get_inverter_realtime_input(inverter.sn)
            if self.logComms:
                indigo.server.log("Grid:")
                indigo.server.log(str(grid)) 
                indigo.server.log("Battery:")
                indigo.server.log(str(battery))
                indigo.server.log("Solar:")
                indigo.server.log(str(solar))           
            device = None
            for dev in indigo.devices.iter("self"):
                if dev.address == inverter.sn:
                    device = indigo.devices[dev.name]
            if device == None:
                indigo.server.log("Creating inverter device for %s" % (inverter.plant.name))
                device = indigo.device.create(protocol=indigo.kProtocol.Plugin, address=inverter.sn, name="Solar Inverter ("+inverter.plant.name+")", pluginId="com.racarter.indigoplugin.sunsynk", deviceTypeId="sunsynkInverter", props={})
            if battery != None:
                device = None
                for dev in indigo.devices.iter("self"):
                    if dev.address == "B"+inverter.sn:
                        device = indigo.devices[dev.name]
                if device == None:
                    indigo.server.log("Creating battery device for %s" % (inverter.plant.name))
                    device = indigo.device.create(protocol=indigo.kProtocol.Plugin, address="B"+inverter.sn, name="Solar Battery ("+inverter.plant.name+")", pluginId="com.racarter.indigoplugin.sunsynk", deviceTypeId="sunsynkBattery", props={})
 
    
    def getReadings(self):
        try:
            inverters = self.client.get_inverters()
            self.retries = 0
        except:
            self.retries += 1
            if self.retries == 10:
                indigo.server.log("Failed to get readings after 10 attempts", isError=True)
                indigo.server.log("Will try again in 10 minutes")
                self.retries = 0
                self.sleep(600)
        if self.logComms:
            indigo.server.log("Inverter:")
            indigo.server.log(str(inverters))
        if inverters == None:
            return
        for inverter in inverters:
            grid = self.client.get_inverter_realtime_grid(inverter.sn)
            battery = self.client.get_inverter_realtime_battery(inverter.sn)
            solar = self.client.get_inverter_realtime_input(inverter.sn)
            if self.logComms:
                indigo.server.log("Grid:")
                indigo.server.log(str(grid)) 
                indigo.server.log("Battery:")
                indigo.server.log(str(battery))
                indigo.server.log("Solar:")
                indigo.server.log(str(solar))          
            for dev in indigo.devices.iter("self"):
                if dev.address == inverter.sn:
                    dev.updateStateOnServer(key="solarWatts", value=solar.get_power(), uiValue=str(solar.get_power())+" W")
                    dev.updateStateOnServer(key="gridWatts", value=grid.get_power(), uiValue=str(grid.get_power())+" W")
                    dev.updateStateOnServer(key="importedToday", value=grid.today_import, uiValue=str(grid.today_import)+" kWh")
                    dev.updateStateOnServer(key="generatedToday", value=inverter.generated_today, uiValue=str(inverter.generated_today)+" kWh")
                    dev.updateStateOnServer(key="generatedTotal", value=inverter.generated_total, uiValue=str(inverter.generated_total)+" kWh")
            for dev in indigo.devices.iter("self"):
                if dev.address == "B"+inverter.sn:
                    dev.updateStateOnServer(key="chargePercent", value=battery.soc, uiValue=str(battery.soc)+"%")
                    dev.updateStateOnServer(key="temperature", value=battery.temp, uiValue=str(battery.temp)+" degC")
                    dev.updateStateOnServer(key="battWatts", value=battery.power, uiValue=str(battery.power)+" W")
                    if int(battery.power) < 0:
                        dev.description = "Charging"
                        dev.replaceOnServer()
                    else:
                        dev.description = ""
                        dev.replaceOnServer()


